require "numru/gave/draw"
require "vtk/Gtk"

GAVE_GTK_VERSION = 2

Gtk.init
Gtk::GL.init

class Draw
  alias :_initialize :initialize
  def initialize(n)
    _initialize(n)
  end
  alias :_clear :clear
  def clear
    _clear
  end
  def open_graphics
    @draw_area = GtkGLExtVTKRenderWindow.new
    @draw_area.set_size_request(@width,@height)
    @window = Gtk::Window.new(Gtk::Window::TOPLEVEL).set_resizable(true)
    @window.signal_connect("delete-event"){|w,p| w.hide; true}
    @window.signal_connect("destroy"){exit}
    @window.add(@draw_area)
    names = @gphys.funcs[0]
    @window.set_title("#{@winno}: #{names[2]}@#{names[0]}")
  end


  private
  def draw_1d(title)
  end

  private
  def draw_2d(title,type)
  end

  def draw_3d(title)
    @fig.figure_set('x = gphys0.coord(0).val', __FILE__,__LINE__)
    @fig.figure_set('x = (x-x.min)/(x.max-x.min)', __FILE__,__LINE__)
    @fig.figure_set('y = gphys0.coord(1).val', __FILE__,__LINE__)
    @fig.figure_set('y = (y-y.min)/(y.max-y.min)', __FILE__,__LINE__)
    @fig.figure_set('z = gphys0.coord(2).val', __FILE__,__LINE__)
    @fig.figure_set('z = (z-z.min)/(z.max-z.min)', __FILE__,__LINE__)
    @fig.figure_set('grid = Vtk::RectilinearGrid.new', __FILE__,__LINE__)
    @fig.figure_set('shape = gphys0.shape', __FILE__,__LINE__)
    @fig.figure_set('grid.SetDimensions(*shape)', __FILE__,__LINE__)
    @fig.figure_set('grid.SetXCoordinates(x.to_va)', __FILE__,__LINE__)
    @fig.figure_set('grid.SetYCoordinates(y.to_va)', __FILE__,__LINE__)
    @fig.figure_set('grid.SetZCoordinates(z.to_va)', __FILE__,__LINE__)
    
    @fig.figure_set('data = gphys0.val', __FILE__,__LINE__)
    @fig.figure_set('data = data.get_array! if NArrayMiss===data', __FILE__,__LINE__)
    @fig.figure_set('data = (data-data.min)/(data.max-data.min)', __FILE__,__LINE__)
    @fig.figure_set('grid.GetPointData.SetScalars(data.reshape!(data.length).to_va)', __FILE__,__LINE__)
    @fig.figure_set('outline = Vtk::OutlineFilter.new', __FILE__,__LINE__)
    @fig.figure_set('outline.SetInput(grid)', __FILE__,__LINE__)
    @fig.figure_set('outlineMapper = Vtk::PolyDataMapper.new', __FILE__,__LINE__)
    @fig.figure_set('outlineMapper.SetInput(outline.GetOutput())', __FILE__,__LINE__)
    @fig.figure_set('outlineActor = Vtk::Actor.new', __FILE__,__LINE__)
    @fig.figure_set('outlineActor.SetMapper(outlineMapper)', __FILE__,__LINE__)
    @fig.figure_set('surface = Vtk::ContourFilter.new', __FILE__,__LINE__)
    @fig.figure_set('surface.SetInput(grid)', __FILE__,__LINE__)
    if @isovalue
      @fig.figure_set('surface.SetValue(0,#{@isovalue})', __FILE__,__LINE__)
    else
      @fig.figure_set('surface.SetValue(0,0.5)', __FILE__,__LINE__)
    end
    @fig.figure_set('normal = Vtk::PolyDataNormals.new', __FILE__,__LINE__)
    @fig.figure_set('normal.SetInput(surface.GetOutput)', __FILE__,__LINE__)
    @fig.figure_set('normal.SetFeatureAngle(45)', __FILE__,__LINE__)
    @fig.figure_set('mapper = Vtk::PolyDataMapper.new', __FILE__,__LINE__)
    @fig.figure_set('mapper.SetInput(normal.GetOutput)', __FILE__,__LINE__)
    @fig.figure_set('mapper.SetScalarModeToUsePointFieldData', __FILE__,__LINE__)
    @fig.figure_set('actor = Vtk::Actor.new', __FILE__,__LINE__)
    @fig.figure_set('actor.SetMapper(mapper)', __FILE__,__LINE__)
    @fig.figure_set('actor.GetProperty.SetOpacity(0.5)', __FILE__,__LINE__)
    @fig.figure_set('ren = Vtk::Renderer.new', __FILE__,__LINE__)
    @fig.figure_set('renWin.GetRenderWindow.AddRenderer(ren)', __FILE__,__LINE__)
    @fig.figure_set('ren.AddActor(outlineActor)', __FILE__,__LINE__)
    @fig.figure_set('ren.AddActor(actor)', __FILE__,__LINE__)
    @fig.figure_set('renWin.set_size_request(500, 500)', __FILE__,__LINE__)
    @fig.figure_set('ren.SetBackground(0.1, 0.1, 0.2)', __FILE__,__LINE__)
    @fig.figure_set('cam1 = ren.GetActiveCamera()', __FILE__,__LINE__)
    @fig.figure_set('cam1.Elevation(-60)', __FILE__,__LINE__)
    @fig.figure_set('cam1.SetViewUp(0, 0, 1)', __FILE__,__LINE__)
    @fig.figure_set('cam1.Azimuth(30)', __FILE__,__LINE__)
    @fig.figure_set('ren.ResetCameraClippingRange()', __FILE__,__LINE__)
    @fig.figure_set('root.show_all', __FILE__,__LINE__)

  end

  private
  def draw(drawing_area)
    if @type == ISOSURFACE
      draw_3d(@title)
    end
    root = @window
    gphys0 = @gphys
    renWin = @draw_area
    codes = @fig.draw_eval
    for j in 0...codes.length
      eval *codes[j]
    end
  end

  private
  class Fig
    attr_reader :head, :var, :open, :close
    def initialize
      @var = ""
      @figure = Array.new
      @head = <<"END"
require "numru/gphys"
require "vtk/Gtk"

include NumRu
END
      @open = <<"END"
Gtk.init
Gtk::GL.init
root = Gtk::Window.new(Gtk::Window::TOPLEVEL)
top = Gtk::VBox.new
root.add(top)
renWin = GtkGLExtVTKRenderWindow.new
top.pack_start(renWin)

END
      @close = <<"END"
Gtk.main
END
    end
    def var_set(funcs,mapping,n)
      @var = "" if n==0
      filename, filetype, varname = funcs[0]
      if filename.length==1
        @var += "filename = \"#{filename[0]}\"\n"
      else
        @var += "filename = [\"#{filename.join('", "')}\"]\n"
      end
      @var += "varname = \"#{varname}\"\n"


      if filetype.nil?
        @var += "gphys#{n} = GPhys::IO.open(filename,varname)\n"
      elsif filetype=="druby"
        @var += "require \"drb/drb\"\n"
        @var += "DRb.start_service\n"
        @var += "class NArray\n"
        @var += "  def self._load(o) to_na(*Marshal::load(o)).ntoh end\n"
        @var += "end\n"
        @var += "ary = filename.split('/')\n"
        @var += "host = 'druby://'+ary[2]\n"
        @var += "path = ary[3..-1].join('/')\n"
        @var += "tree = DRbObject.new(nil,host)\n"
        @var += "gphys#{n} = tree.data(path)\n"
      else
        raise "filetype is invalid"
      end

      if funcs.length!=0
	funcs[1..-1].each{|func|
	  @var += "gphys#{n} = gphys#{n}.cut(#{func[2].join(",")})\n"
	  @var += "gphys#{n} = gphys#{n}.#{func[0]}(#{func[1]})\n"
	}
      end
      str = mapping.collect{|a|
        if NArray===a
          "NArray[#{a.to_a.join(",")}]"
        else
          a
        end
        }.join(",")
      @var += "gphys#{n} = gphys#{n}[#{str}]\n"
    end

    def title_set(xt,xu,yt,yu, file,line)
    end

    def figure_clear
      @figure.clear
    end
    def figure_set(str,file,line)
      @figure.push([str,file,line])
    end
    def draw_parm
    end
    def draw
      @figure.join("\n\n")
    end
    def draw_eval
      codes = Array.new
      @figure.each{|c| codes.push [c[0], nil, c[1], c[2]]}
      codes
    end
  end
end
